package mil.af.afit.router.app;
import mil.af.afit.router.*;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

public class MainPanel extends JPanel implements Observer
{
    private final static String
        NO_SOLUTIONS_TEXT = "no solutions",
        TT_TOTAL_SITES = "Total number of sites that are loaded",
        TT_ENABLED_SITES = "Number of sites the vehicles will try to visit",
        TT_TOTAL_VEHICLES = "Total number of vehicles that are loaded",
        TT_ENABLED_VEHICLES = "Number of vehicles that will try to visit sites",
        TT_LINE1_OTHER = "Wind speed in knots",
        TT_LINE2_OTHER = "Wind bearing in degrees clockwise from North",
        TT_SHORTEST_MISSION = "Measured from the first takeoff to the last landing",
        TT_SHORTEST_MISSION_SITES_VISITED = "Number of sites that get visited on the shortest mission",
        TT_SITES_BUTTON = "View/edit information about all the sites",
        TT_VEHICLES_BUTTON = "View/edit information about vehicles and bases",
        TT_OTHER_BUTTON = "View/edit miscellaneous information and settings",
        TT_SOLVE_BUTTON = "Try to find a way to make the vehicles visit all the sites",
        TT_MORE_SOLN_BUTTON = "View known solutions",
        TT_LAF = "Change the look and feel";
    
    Kernel kernel = null;
    // These declarations are borrowed from the code generated by
    // Symantec's Visual Cafe which was used to rapidly develop
    // the look and feel of this panel.
	JPanel pnlTop = new JPanel();
	    JPanel pnlSites = new JPanel();
	        JPanel pnlSitesLabels = new JPanel();
	            JPanel pnlTotalSitesLabels = new JPanel();
	                JLabel lblTotalSites = new JLabel();
	                JLabel lblTotalSitesValue = new JLabel();
	            JPanel pnlEnabledSitesLabels = new JPanel();
	                JLabel lblEnabledSites = new JLabel();
	                JLabel lblEnabledSitesValue = new JLabel();
	        JPanel pnlSitesButton = new JPanel();
	            JButton btnSites = new JButton();
	    JPanel pnlVehicles = new JPanel();
	        JPanel pnlVehLabels = new JPanel();
	            JPanel pnlTotalVehiclesLabels = new JPanel();
	                JLabel lblTotalVehicles = new JLabel();
	                JLabel lblTotalVehiclesValue = new JLabel();
	            JPanel pnlEnabledVehiclesLabels = new JPanel();
	                JLabel lblEnabledVehicles = new JLabel();
	                JLabel lblEnabledVehiclesValue = new JLabel();
	        JPanel pnlVehButton = new JPanel();
	            JButton btnVehicles = new JButton();

	
	    JPanel pnlOther = new JPanel();
	        JPanel pnlOtherLabels = new JPanel();
	            JPanel pnlLine1OtherLabels = new JPanel();
	                JLabel lblLine1Other = new JLabel();
	                JLabel lblLine1OtherValue = new JLabel();
	            JPanel pnlLine2OtherLabels = new JPanel();
	                JLabel lblLine2Other = new JLabel();
	                JLabel lblLine2OtherValue = new JLabel();
	        JPanel pnlOtherButton = new JPanel();
	            JButton btnOther = new JButton();
	
	
	
	JButton btnSolve = new JButton();
	JPanel pnlSolutions = new JPanel();
	    JPanel pnlSolutionLabels = new JPanel();
	        JPanel pnlMissionLengthLabels = new JPanel();
	            JLabel lblShortestMissionLength = new JLabel();
	            JLabel lblShortestMissionLengthValue = new JLabel();
	        JPanel pnlMissionSitesVisitedLabels = new JPanel();
	            JLabel lblShortestMissionSitesVisited = new JLabel();
	            JLabel lblShortestMissionSitesVisitedValue = new JLabel();
	    JPanel pnlSolutionButtons = new JPanel();
	        JButton btnMoreSolutions = new JButton();
	        JButton btnSaveSolutions = new JButton();

    
    
    public MainPanel( Kernel kernel )
    {   
        this.kernel = kernel;
        this.setupLayout();
        this.setupLinksWithKernel();
        this.setupActionListeners();
    }   // end constructor
    
    private final void setupLayout()
    {
        // This initialization code is borrowed from the code generated by
        // Symantec's Visual Cafe which was used to rapidly develop
        // the look and feel of this panel.
        GridBagLayout gbl = new GridBagLayout();
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.weightx = gbc.weighty = 1;
		this.setLayout( gbl );
		pnlTop.setLayout(new GridLayout(1,3,0,0));
		gbc.gridx = gbc.gridy = 0;
		gbc.fill = gbc.BOTH;
		gbl.setConstraints( pnlTop, gbc );
		this.add( pnlTop );//, new GridBagConstraints(0,0,1,1,1.0,0.0,GridBagConstraints.CENTER,GridBagConstraints.BOTH,new Insets(7,5,0,5),0,0) );
		    pnlTop.add(pnlSites);
                pnlSites.setBorder( new TitledBorder( "Sites" ) );
		        pnlSites.setLayout(new GridLayout(2,1));
		        pnlSites.add(pnlSitesLabels);
		            pnlSitesLabels.setLayout(new GridLayout(2,1));
		            pnlSitesLabels.add(pnlTotalSitesLabels);
		                pnlTotalSitesLabels.setToolTipText( TT_TOTAL_SITES );
		                pnlTotalSitesLabels.setLayout(new BorderLayout(0,0));
		                pnlTotalSitesLabels.add("West", lblTotalSites);
		                    lblTotalSites.setText("Total:");
		                    lblTotalSites.setLabelFor(lblTotalSitesValue);
		                pnlTotalSitesLabels.add("East", lblTotalSitesValue);
		                    lblTotalSitesValue.setHorizontalAlignment(SwingConstants.RIGHT);
		                    lblTotalSitesValue.setText("000");
		            pnlSitesLabels.add(pnlEnabledSitesLabels);
		                pnlEnabledSitesLabels.setToolTipText( TT_ENABLED_SITES );
		                pnlEnabledSitesLabels.setLayout(new BorderLayout(0,0));
		                pnlEnabledSitesLabels.add("West", lblEnabledSites);
		                    lblEnabledSites.setText("Enabled:");
		                    lblEnabledSites.setLabelFor(lblEnabledSitesValue);
		                pnlEnabledSitesLabels.add("East", lblEnabledSitesValue);
		                    lblEnabledSitesValue.setHorizontalAlignment(SwingConstants.RIGHT);
		                    lblEnabledSitesValue.setText("000");
		        pnlSites.add(pnlSitesButton);
		            pnlSitesButton.setLayout(new FlowLayout(FlowLayout.CENTER,5,5));
		            pnlSitesButton.add(btnSites);
		                btnSites.setText("Sites...");
		                btnSites.setActionCommand("Sites...");
		                btnSites.setToolTipText( TT_SITES_BUTTON );

		    pnlTop.add(pnlVehicles);
		        pnlVehicles.setBorder(new TitledBorder( "Vehicles" ));
		        pnlVehicles.setLayout(new GridLayout(2,1));
		        pnlVehicles.add(pnlVehLabels);
		            pnlVehLabels.setLayout(new GridLayout(2,1));
		            pnlVehLabels.add(pnlTotalVehiclesLabels);
		                pnlTotalVehiclesLabels.setToolTipText( TT_TOTAL_VEHICLES );
		                pnlTotalVehiclesLabels.setLayout(new BorderLayout(0,0));
		                pnlTotalVehiclesLabels.add("West", lblTotalVehicles);
		                    lblTotalVehicles.setText("Total:");
		                    lblTotalVehicles.setLabelFor(lblTotalVehiclesValue);
		                pnlTotalVehiclesLabels.add("East", lblTotalVehiclesValue);
		                    lblTotalVehiclesValue.setHorizontalAlignment(SwingConstants.RIGHT);
		                    lblTotalVehiclesValue.setText("000");
		            pnlVehLabels.add(pnlEnabledVehiclesLabels);
		                pnlEnabledVehiclesLabels.setToolTipText( TT_ENABLED_VEHICLES );
		                pnlEnabledVehiclesLabels.setLayout(new BorderLayout(0,0));
		                pnlEnabledVehiclesLabels.add("West", lblEnabledVehicles);
		                    lblEnabledVehicles.setText("Enabled:");
		                    lblEnabledVehicles.setLabelFor(lblEnabledVehiclesValue);
		                pnlEnabledVehiclesLabels.add("East", lblEnabledVehiclesValue);
		                    lblEnabledVehiclesValue.setHorizontalAlignment(SwingConstants.RIGHT);
		                    lblEnabledVehiclesValue.setText("000");
		        pnlVehicles.add(pnlVehButton);
		            pnlVehButton.setLayout(new FlowLayout(FlowLayout.CENTER,5,5));
		            pnlVehButton.add(btnVehicles);
		                btnVehicles.setText("Vehicles...");
		                btnVehicles.setActionCommand("Vehicles...");
		                btnVehicles.setToolTipText( TT_VEHICLES_BUTTON );

		    pnlTop.add(pnlOther);
                pnlOther.setBorder( new TitledBorder( "Other" ) );
		        pnlOther.setLayout(new GridLayout(2,1));
		        pnlOther.add(pnlOtherLabels);
		            pnlOtherLabels.setLayout(new GridLayout(2,1));
		            pnlOtherLabels.add(pnlLine1OtherLabels);
		                pnlLine1OtherLabels.setToolTipText( TT_LINE1_OTHER );
		                pnlLine1OtherLabels.setLayout(new BorderLayout(0,0));
		                pnlLine1OtherLabels.add("West", lblLine1Other);
		                    lblLine1Other.setText("Wind speed:");
		                    lblLine1Other.setLabelFor(lblLine1OtherValue);
		                pnlLine1OtherLabels.add("East", lblLine1OtherValue);
		                    lblLine1OtherValue.setHorizontalAlignment(SwingConstants.RIGHT);
		                    lblLine1OtherValue.setText("000");
		            pnlOtherLabels.add(pnlLine2OtherLabels);
		                pnlLine2OtherLabels.setToolTipText( TT_LINE2_OTHER );
		                pnlLine2OtherLabels.setLayout(new BorderLayout(0,0));
		                pnlLine2OtherLabels.add("West", lblLine2Other);
		                    lblLine2Other.setText("Wind from:");
		                    lblLine2Other.setLabelFor(lblLine2OtherValue);
		                pnlLine2OtherLabels.add("East", lblLine2OtherValue);
		                    lblLine2OtherValue.setHorizontalAlignment(SwingConstants.RIGHT);
		                    lblLine2OtherValue.setText("UNKNOWN DIRECTION");
		        pnlOther.add(pnlOtherButton);
		            pnlOtherButton.setLayout(new FlowLayout(FlowLayout.CENTER,5,5));
		            pnlOtherButton.add(btnOther);
		                btnOther.setText("Other...");
		                btnOther.setActionCommand("Other...");
		                btnOther.setToolTipText( TT_OTHER_BUTTON );
		
		
		
		gbc.gridy = 1;
		gbc.fill = gbc.BOTH;
		gbl.setConstraints( pnlSolutions, gbc );
		this.add( pnlSolutions );//, new GridBagConstraints(0,2,1,1,1.0,0.0,GridBagConstraints.CENTER,GridBagConstraints.BOTH,new Insets(7,5,0,5),0,0) );
		    pnlSolutions.setBorder(new TitledBorder( "Solutions" ));
		    pnlSolutions.setLayout(new BorderLayout(0,0));
		    pnlSolutions.add("Center", pnlSolutionLabels);
		        pnlSolutionLabels.setLayout(new GridLayout(2,1));
		        pnlSolutionLabels.add(pnlMissionLengthLabels);
		            pnlMissionLengthLabels.setToolTipText( TT_SHORTEST_MISSION );
		            pnlMissionLengthLabels.setLayout(new BorderLayout(0,0));
		            pnlMissionLengthLabels.add("West", lblShortestMissionLength);
		                lblShortestMissionLength.setText("Shortest mission, length:");
		                lblShortestMissionLength.setLabelFor(lblTotalSitesValue);
		            pnlMissionLengthLabels.add("East", lblShortestMissionLengthValue);
		                lblShortestMissionLengthValue.setHorizontalAlignment(SwingConstants.RIGHT);
		                lblShortestMissionLengthValue.setText("NO_SOLUTIONS_TEXT");
		        pnlSolutionLabels.add(pnlMissionSitesVisitedLabels);
		            pnlMissionSitesVisitedLabels.setToolTipText( TT_SHORTEST_MISSION_SITES_VISITED );
		            pnlMissionSitesVisitedLabels.setLayout(new BorderLayout(0,0));
		            pnlMissionSitesVisitedLabels.add("West", lblShortestMissionSitesVisited);
		                lblShortestMissionSitesVisited.setText("Shortest mission, sites visited:");
		                lblShortestMissionSitesVisited.setLabelFor(lblEnabledSitesValue);
		            pnlMissionSitesVisitedLabels.add("East", lblShortestMissionSitesVisitedValue);
		                lblShortestMissionSitesVisitedValue.setHorizontalAlignment(SwingConstants.RIGHT);
		                lblShortestMissionSitesVisitedValue.setText("NO_SOLUTIONS_TEXT");
		    pnlSolutions.add("South", pnlSolutionButtons);
		        pnlSolutionButtons.setLayout( new BorderLayout() );
		        JPanel pnlSolutionButtons2 = new JPanel( new FlowLayout(FlowLayout.CENTER,15,5) );
		        pnlSolutionButtons.add( pnlSolutionButtons2, BorderLayout.CENTER );
		        pnlSolutionButtons2.add(btnSolve);
		            btnSolve.setText("Solve...");
		            btnSolve.setActionCommand("Solve...");
		            btnSolve.setToolTipText( TT_SOLVE_BUTTON );
		        pnlSolutionButtons2.add(btnMoreSolutions);
		            btnMoreSolutions.setText("More...");
		            btnMoreSolutions.setActionCommand("More...");
		            btnMoreSolutions.setToolTipText( TT_MORE_SOLN_BUTTON );
		        // L&F Menu
		        final JLabel lafLabel = new JLabel("L&F",JLabel.RIGHT);
		        lafLabel.setVerticalAlignment( JLabel.BOTTOM );
		        lafLabel.setToolTipText( TT_LAF );
                JLabel blank = new JLabel("   ");
                blank.setSize( lafLabel.getSize() );
                blank.setMinimumSize( blank.getSize() );
		        pnlSolutionButtons.add( blank, BorderLayout.WEST );
		        pnlSolutionButtons.add( lafLabel, BorderLayout.EAST );
                lafLabel.addMouseListener( new MouseAdapter()
                {   public void mouseClicked( MouseEvent e )
                    {   new LAFPopupMenu().show( lafLabel, e.getX(), e.getY() );
                    }   // end mouseClicked
                }); // end mouseadapter

    
    
    }   // end setupLayout
    
    
    
/* ********  D I S P O S E   C O D E  ******** */


    public final void dispose()
    {   this.kernel.deleteObserver( this );
    }   // end dispose
    
    
/* ********  K E R N E L   A C C E S S   C O D E  ******** */
    
    
    private final synchronized void setupLinksWithKernel()
    {
        // Register self to observe the kernel
        kernel.addObserver( this );
        
        // Force an update of relevant components
        updateSiteStats( kernel.getSiteList() );
        updateVehicleStats( kernel.getVehicleList() );
        updateSolutionStats( kernel.getSolutionList() );
        updateWindStats( kernel.getWindList() );
    }   // end setupLinksWithKernel
    
    public final synchronized void setKernel( Kernel kernel )
    {
        // If we already had a kernel, remove self as observer
        if( this.kernel != null )
            this.kernel.deleteObserver( this );
        this.kernel = kernel;
        setupLinksWithKernel();
    }   // end setKernel
    
    
    private final void setupActionListeners()
    {
        // Add appropriate actions to buttons
        this.addActionListenerToSitesButton(
            new OpenSitesFrameAction( kernel ) );
        this.addActionListenerToVehiclesButton(
            new OpenVehicleBaseFrameAction( kernel ) );
        this.addActionListenerToSolveButton( 
            new OpenSolveDialogAction( kernel, this ) );
        this.addActionListenerToSolutionsMoreButton(
            new OpenSolutionsFrameAction( kernel ) );
        this.addActionListenerToOtherButton( 
            new OpenOtherSettingsDialogAction( kernel, this ) );
        
        
    }   // end setupActionListener
    
    
    
/* ********  A C T I O N   A C C E S S   C O D E  ******** */
    
    public final void addActionListenerToOtherButton( ActionListener listener )
    {   btnOther.addActionListener( listener );
    }   // end addActionListenerToOtherButton
    
    public final void addActionListenerToSitesButton( ActionListener listener )
    {   btnSites.addActionListener( listener );
    }   // end addActionListenerToSitesButton
    
    public final void addActionListenerToVehiclesButton( ActionListener listener )
    {   btnVehicles.addActionListener( listener );
    }   // end addActionListenerToVehiclesButton
    
    public final void addActionListenerToSolveButton( ActionListener listener )
    {   btnSolve.addActionListener( listener );
    }   // end addActionListenerToSolveButton
    
    public final void addActionListenerToSolutionsMoreButton( ActionListener listener )
    {   btnMoreSolutions.addActionListener( listener );
    }   // end addActionListenerToSolutionsMoreButton
    
   // public final void addActionListenerToSolutionsSaveButton( ActionListener listener )
   // {   btnSaveSolutions.addActionListener( listener );
   // }   // end addActionListenerToSolutionsSaveButton
    
    public final void removeActionListenerFromSitesButton( ActionListener listener )
    {   btnSites.removeActionListener( listener );
    }   // end removeActionListenerFromSitesButton
    
    public final void removeActionListenerFromVehiclesButton( ActionListener listener )
    {   btnVehicles.removeActionListener( listener );
    }   // end removeActionListenerFromVehiclesButton
    
    public final void removeActionListenerFromSolveButton( ActionListener listener )
    {   btnSolve.removeActionListener( listener );
    }   // end removeActionListenerFromSolveButton
    
    public final void removeActionListenerFromSolutionsMoreButton( ActionListener listener )
    {   btnMoreSolutions.removeActionListener( listener );
    }   // end removeActionListenerFromSolutionsMoreButton
    
    //public final void removeActionListenerFromSolutionsSaveButton( ActionListener listener )
    //{   btnSaveSolutions.removeActionListener( listener );
    //}   // end removeActionListenerFromSolutionsSaveButton
    
    
    
/* ********  S T A T S   C O D E  ******** */
    
    private final void updateSiteStats( SiteList siteList )
    {   final int total = siteList.getSites().length;
        final int enabled = siteList.countEnabledSites();
        final JPanel This = this;
        SwingUtilities.invokeLater( new Runnable()
        {   public void run()
            {   lblTotalSitesValue.setText( ""+total );
                lblEnabledSitesValue.setText( ""+enabled );
                This.validate();
            }   // end run
        }); // end runnable, invokeLater
    }   // end updateSiteStats
    
    private final void updateVehicleStats( VehicleList vehicleList )
    {   final int total = vehicleList.getVehicles().length;
        final int enabled = vehicleList.countEnabledVehicles();
        final JPanel This = this;
        SwingUtilities.invokeLater( new Runnable()
        {   public void run()
            {   lblTotalVehiclesValue.setText( ""+total );
                lblEnabledVehiclesValue.setText( ""+enabled );
                This.validate();
            }   // end run
        }); // end runnable, invokeLater
    }   // end updateVehicleStats
    
    
    private final void updateWindStats( WindList windList )
    {   Wind[] winds = windList.getWinds();
        if( winds.length >= 1 )
        {
            Wind wind = winds[0]; // We're just using the first wind in this version.
            final int speed = wind.getSpeed();
            final int bearing = wind.getBearing();
            
            // Determine a source based on the bearing. Note that we're
            // finding where the wind is coming FROM. Use a method defined
            // below this one to determine if an int is between two numbers, inclusively.
            /*
                LEGEND: Numbers are the bearing. Characters represent where
                        the wind is coming FROM.
                        
            
                                 337\338    22/23  
                                     \   S   /
                           293    SE  \     /  SW    67
                          292--__      \   /      __--68
                                 ---___ \ / ___---    
                              E     ___=-+-=___     W
                               __---    / \    ---__
                          248--        /   \        --112
                          247    NE   /  N  \   NW    113
                                     /       \
                                 203/202   158\157   
            
            
            */
            String direction = "";
            
            if( isBetween(bearing, 338, 360) || isBetween(bearing,0,22) )
                direction = "South";
            else if( isBetween(bearing,23,67) )
                direction = "Southwest";
            else if( isBetween(bearing,68,112) )
                direction = "West";
            else if( isBetween(bearing,113,157) )
                direction = "Northwest";
            else if( isBetween(bearing,158,202) )
                direction = "North";
            else if( isBetween(bearing,203,247) )
                direction = "Northeast";
            else if( isBetween(bearing,248,292) )
                direction = "East";
            else if( isBetween(bearing,293,337) )
                direction = "Southeast";
                
            final String windFrom = direction;
            final JPanel This = this;
            SwingUtilities.invokeLater( new Runnable()
            {   public void run()
                {   lblLine1OtherValue.setText( ""+speed );
                    lblLine2OtherValue.setText( windFrom );
                    This.validate();
                }   // end run
            }); // end runnable, invokeLater
        }   // end if: at least one wind
    }   // end updateWindStats
    
    private final boolean isBetween( int x, int lower, int upper )
    {   return ( (x >= lower) && (x <= upper) ) ? true : false;
    }   // end isBetween
    
    
    private final void updateSolutionStats( SolutionList solutionList )
    {   
        final Solution[] solutions = solutionList.getSolutions();
        if( solutions.length > 0 )
        {
            int length = Integer.MAX_VALUE;
            int visited = -Integer.MAX_VALUE;
            int indexOfShortestSolution = 0;
            
            for( int i = 0; i < solutions.length; i++ )
            {   if( solutions[i].getMissionLength() < length )
                {   length = solutions[i].getMissionLength();
                    visited = solutions[i].countSitesVisited();
                    indexOfShortestSolution = i;
                }   // end if: new shorter mission
            }   // end for: through each solution
            
            // Prepare for anonymous inner class
            final JPanel This = this;
            final int Length = length;
            final int Visited = visited;
            final JLabel LengthLabel = lblShortestMissionLengthValue;
            final JLabel VisitedLabel = lblShortestMissionSitesVisitedValue;
            SwingUtilities.invokeLater( new Runnable()
            {   public void run()
                {   LengthLabel.setText( ""
                        + (Length/60) + " hrs. "
                        + (Length%60) + " min." );
                    VisitedLabel.setText( ""+Visited );
                    This.validate();
                }   // end run
            }); // end runnable, invokeLater
        }   // end if: at least one solution
        else
        {
            final MainPanel This = this;
            SwingUtilities.invokeLater( new Runnable()
            {   public void run()
                {   lblShortestMissionLengthValue.setText(NO_SOLUTIONS_TEXT);
                    lblShortestMissionSitesVisitedValue.setText(NO_SOLUTIONS_TEXT);
                    This.validate();
                }   // end run
            }); // end runnable, invokeLater
        }   // end else: no solutions
    }   // end updateSolutionStats
    
    
    
    
/* ********  O B S E R V E R   C O D E  ******** */
    
    public void update( Observable thing, Object arg )
    {
        if( arg instanceof SiteList )
            updateSiteStats( (SiteList) arg );
        else if( arg instanceof VehicleList )
            updateVehicleStats( (VehicleList) arg );
        else if( arg instanceof SolutionList )
            updateSolutionStats( (SolutionList) arg );
        else if( arg instanceof WindList )
            updateWindStats( (WindList) arg );
    }   // end update
    
}   // end class MainPanel